#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#define max(a,b) (((a)>(b))?(a):(b))
#define min(a,b) (((a)<(b))?(a):(b))

typedef struct {
  float duration; // of the power sample in seconds
  float power; // average, in watts
} resample_t;

const float resample_period=1.0; // seconds

float *resample(resample_t input[], int count, int *outcount) {
  int in_i,out_i;
  float duration = 0.0;

  for (in_i=0; in_i<count; in_i++) { 
    duration += input[in_i].duration;
  }
  
  int output_bins = ceilf(duration / resample_period);
  float* ret = (float *)calloc(output_bins,sizeof(float));
  
  float out_starttime=0.0;
  float in_starttime=0.0;
  
  for (in_i=0, out_i=0; (in_i<count) && (out_i<output_bins); ) {

    float in_endtime = in_starttime+input[in_i].duration;
    float out_endtime = out_starttime+resample_period;
    
    float overlap_start = max(in_starttime, out_starttime);
    float overlap_end = min(in_endtime, out_endtime);
    
    float overlap_duration = overlap_end - overlap_start;

    assert(overlap_duration >= 0);

    ret[out_i] += input[in_i].power * overlap_duration / resample_period;
   
    // move along appropriate pointers
    if (in_endtime < out_endtime) in_i++, in_starttime = in_endtime;
    else out_i++, out_starttime = out_endtime;
  }

  *outcount=output_bins;
  return ret;
}

int main(void) {
  
  resample_t in_samples[100];
  float *result;
  int outcount;

  // warning: floating point equality testing ahead...

#if 1
  printf("test: 120 RPM, 100/300 W squarewave... ");

  in_samples[0].duration=0.5;
  in_samples[0].power=100.0;
  in_samples[1].duration=0.5;
  in_samples[1].power=300.0;

  result=resample(in_samples,2,&outcount);  

  assert(outcount==1);
  assert(result[0]==200.0);
  free(result);
  printf("pass\n");
#endif 

#if 1
  printf("test: 20 RPM, 100/300 W squarewave... ");

  in_samples[0].duration=3;
  in_samples[0].power=100.0;
  in_samples[1].duration=3;
  in_samples[1].power=300.0;

  result=resample(in_samples,2,&outcount);  

  assert(outcount==6);
  assert(result[0]==100.0);
  assert(result[1]==100.0);
  assert(result[2]==100.0);
  assert(result[3]==300.0);
  assert(result[4]==300.0);
  assert(result[5]==300.0);
  free(result);
  printf("pass\n");
#endif

  printf("all pass\n");
  return 0;
}
